#ifndef WIRE_H
#define WIRE_H

#include "logdef.h"

typedef struct log_hwrec {
  short x1, x2, y;   /* Position of wire */
  log_nrec *node;   /* Node of wire */
  struct log_hwrec *next;   /* Next wire in list */
  na_long temp;
  uchar wcolr;   /* Color of wire */
  unsigned f7 : 1, f6 : 1, f5 : 1, f4 : 1, f3 : 1, f2 : 1, f1 : 1, f0 : 1;
} log_hwrec;   /* Horizontal wire */

typedef struct log_vwrec {
  short x, y1, y2;   /* Position of wire */
  log_nrec *node;   /* Node of wire */
  struct log_vwrec *next;   /* Next wire in list */
  na_long temp;
  uchar wcolr;   /* Color of wire */
  unsigned f7 : 1, f6 : 1, f5 : 1, f4 : 1, f3 : 1, f2 : 1, f1 : 1, f0 : 1;
} log_vwrec;   /* Vertical wire */

typedef struct log_srec {
  short x, y;   /* Position of solder point */
  log_hwrec *hwire, *hwire2;   /* Horizontal wire(s) being soldered */
  log_vwrec *vwire, *vwire2;   /* Vertical wire(s) being soldered */
  struct log_srec *next;   /* Next solder point in list */
  na_long temp;
  unsigned f7 : 1, f6 : 1, f5 : 1, f4 : 1, f3 : 1, f2 : 1, f1 : 1, f0 : 1;
} log_srec;   /* Solder point */

void newhw(log_hwrec **hw);

void newvw(log_vwrec **vw);

void addvwire(short x1, short x2, short y, short colr);

void addhwire(short x1, short x2, short y, short colr);

void addvwire2(short x, short y1, short y2);

void addhwire2(short x1, short x2, short y);

void delvwire(log_vwrec *vw);

void delhwire(log_hwrec *hw);

void closerwire(short x, short y);

void newsolder(log_srec **s);

void addsolder(short x, short y, log_hwrec *hw, log_hwrec *hw2, log_vwrec *vw, log_vwrec *vw2);

log_srec *findsolder(short x, short y);

void addsoldert(log_hwrec *hw, log_vwrec *vw);

void chghw(log_hwrec *hw, log_nrec *oldnode, log_nrec *n);

void chgvw(log_vwrec *vw, log_nrec *oldnode, log_nrec *n);

void unsoldernear();

void unsolderwires(log_hwrec *hw, log_vwrec *vw);

void soldernear();

void solderat(short x, short y);

#endif
