#include "pen.h"

#include <graphics/newci.h>
#include <graphics/newcrt.h>
#include "logdef.h"
#include "logglobals.h"
#include "keyboard.h"
#include "log.h"
#include "window.h"

/** @brief Find the position of the pen.
			Also draws the cursor, runs logic probe.

	Returns:
		- D, D0 if the pen is/was down;
		- N, N0 if the pen is/was near;
		- DN, UP if the pen is pressed/released;
		- NR if pen remains near;
		- OFFSCREEN if pen is off the screen edge;
		- PX, PY, PX0, PY0 = new/old position;
		- FX, FY = position in grid coordinates;
		- MENUBOX = menu box number (0-15);
*/
void pen()
{

#define ALERTNESS 2 /* Time to get bored (in cs) */
#define DOZETIME  5 /* Time to sleep after nodding off */

	long x, y;
	short i, rx, ry, cred;
	/* static int pollkbd2(void); */
	static int oldx, oldy;
	static long awake;
	long now;
	char rval, gval, bval;


	/* the following section improves idling behavior       */
	/* contributed by Nick Bailey (een6njb@sun.leeds.ac.uk) */

	/* Keep on your toes if s/he's still twitching about! */
	now = timers_sysclock();
	if (gg.t.x!=oldx || gg.t.y!=oldy || pollkbd2() || gg.busyflag)
	{
		awake = now;
		oldx = gg.t.x; oldy = gg.t.y;
	}
	/* Be friendly when not running something important */
	if (now-awake > (unsigned long)ALERTNESS)
		microsleep((unsigned long)10000*DOZETIME);

	/* end of idling-improvement section */

	TRY(try3);
	gg.t0 = gg.t;
	m_readpen(&gg.t);
	show_events();
	gg.stillnear = (gg.stillnear && gg.t.near_);
	gg.incircuit = (gg.t.y < baseline && gg.showpage > 0);
	RECOVER(try3);
	if (P_escapecode == -20)
		_Escape(P_escapecode);
	nc_printf("Graphics tablet error\n");
	m_init_pen(tabletaddr);
	m_alpha_on();
	ENDTRY(try3);
	if (snapflag && gg.incircuit)
	{
		gg.t.x = (gg.t.x + gg.hscale + gg.xoff) / gg.scale * gg.scale - gg.xoff;
		gg.t.y = (gg.t.y + gg.hscale + gg.yoff) / gg.scale * gg.scale - gg.yoff;
	}

	if (gg.t.x < 0)
		gg.t.x = 0;
	else if (gg.t.x > across)
		gg.t.x = across;

	if (gg.t.y < 0)
		gg.t.y = 0;
	else if (gg.t.y > down)
		gg.t.y = down;

	if (gg.t.near_ && !gg.t.depressed)
	{
		rx = gg.t.x * 5 / across;
		ry = gg.t.y * 5 / down;
	}

	if (gg.probemode)
	{
		short cursx = getcursorxposition();
		short cursy = getcursoryposition();
		if (cursx != gg.t.x || cursy != gg.t.y)
		{
			if (gg.probesimtype != NULL)
				checkprobe((int)gg.t.x, (int)gg.t.y);
			probeflag = false;
			drawcursor((int)gg.t.x, (int)gg.t.y);
		}
		else
		{
			if (!getcursorflag())
				drawcursor((int)gg.t.x, (int)gg.t.y);
			if (!probeflag)
			{
				testprobe((int)gg.t.x, (int)gg.t.y);
				probeflag = true;
			}
		}

		if (!gg.t.near_ && briefprobe)
		{
			remcursor();
			gg.probemode = false;
		}
	}
	else
	{
		briefprobe = false;
		if (gg.probesimtype != NULL)
			unprobe();
		if (gg.t.near_)
			drawcursor((int)gg.t.x, (int)gg.t.y);
		else
			remcursor();
	}
	cred = gg.color.conflict;
	if (anyconflicts && (cred == log_cred || cred == log_cred8) &&
			(rcolormap[cred] == 255 || gcolormap[cred] == 255 ||
			 bcolormap[cred] == 255))
	{
		x = timers_sysclock() * 6 % 137 + 119;
		rval = (char)(x * rcolormap[cred] / 255);
		gval = (char)(x * gcolormap[cred] / 255);
		bval = (char)(x * bcolormap[cred] / 255);
		m_vsetcolors((long)cred, 1L, (unsigned char *) &rval,
				(unsigned char *) &gval, (unsigned char *) &bval);
	}

	if (gg.t.moving)
	{
		gg.fastspeed = gg.fastmin;
		fastsavetime = timers_sysclock();
	}
	else if (gg.fastspeed < gg.fastmax &&
			timers_sysclock() > fastsavetime + gg.fastrate)
	{
		gg.fastspeed++;
		fastsavetime = timers_sysclock();
	}
	gg.gridx = (gg.t.x + gg.hscale + gg.xoff) / gg.scale;
	gg.gridy = (gg.t.y + gg.hscale + gg.yoff) / gg.scale;
}


///  Return TRUE if pen was tapped. Return FALSE if pen was held and/or moved.
int justtap()
{
	long t0;
	short tx, ty;

	t0 = timers_sysclock() + tapdelay;
	tx = gg.t.x;
	ty = gg.t.y;
	do
	{
		pen();
	} while (gg.t.depressed && labs(gg.t.x - tx) <= taptolerance &&
			labs(gg.t.y - ty) <= taptolerance && timers_sysclock() <= t0);
	return (!gg.t.depressed);
}


/// Return TRUE if cursor is inside a certain rectangular area (screen coordinates).
int inbox(short x, short y, short x1, short y1)
{
	return (gg.t.x >= x && gg.t.y >= y && gg.t.x <= x + x1 && gg.t.y <= y + y1);
}

