#define MISC_G

#include <stdio.h>
#include <string.h>
#include <utils/p2c.h>
#include <utils/misc.h>

#ifndef LACK_GETPWNAM
#include <pwd.h>
#endif

void misc_getioerrmsg(char *s, long i)
{
	sprintf(s, "misc_getioerrmsg Error number %ld", i);
}

#if defined __FreeBSD__ || defined __bsdi__

char *cuserid(char *s)
{
	char *sp;
	static char sbuf[L_cuserid + 1];

	if ((sp = getenv("USER")) == NULL) {
		if (s != NULL)
			s[0] = '\0';
		return NULL;
	}
	return strncpy(s != NULL ? s : sbuf, sp, L_cuserid);
}

#endif


#ifndef LACK_GETPWNAM

/* tilde-expansion for platforms with passwd mechanisms.
 * this replaces a leading ~/ with the user's home directory,
 * or, ~foo with foo's home directory, if foo is a recognised user.
 */
char *tilde_expand(char *s)
{
	char *name;
	char *ret = NULL;
	size_t len = 0;
	struct passwd *pw;

	if (*s == '~') {
		if (strlen(s) > 1)
			len = strcspn(s+1, "/");
		if (len > 0) {
			name = (char *) Malloc(len + 1);
			strncpy(name, s+1, len);
			name[len] = '\0';
			pw = getpwnam(name);
			Free(name);
		} else {
			pw = getpwuid( geteuid() );
		}
		if (pw != NULL) {
			ret = (char *) Malloc(strlen(pw->pw_dir) + strlen(s) - len);
			strcpy(ret,pw->pw_dir);
			if (strlen(s) > len + 1)
				strcat(ret, s + len + 1);
		}
	}
	if (ret == NULL)
		ret = strdup(s);
	return ret;
}

#else

/* simple tilde expansion for platforms without usernames/password mechanisms.
 * replace a leading '~' with the value of the $HOME environment variable.
 */
char *tilde_expand(char *s)
{
	char *hp;
	char *ret = NULL;

	if (*s == '~' && (hp = getenv("HOME")) != NULL) {
		if ((ret = (char *) Malloc(strlen(hp) + strlen(s))) != NULL) {
			strcpy(ret, hp);
			if (strlen(s) > 1)
				strcat(ret, s + 1);
		}
	}
	if (ret == NULL)
		ret = strdup(s);
	return ret;
}

#endif
