/* caged_process='cc -c $F$M'  caged_tabs=4 */


/* C version of the Caltech NEWASM library */
/* Dave Gillespie, 2-14-88 */


#define KEEP_P2C_DECLS

# include <time.h>

#include <utils/p2c.h>
#include <utils/newasm.h>

/* The following would benefit from assembly language if possible */

#define ROL_L(v,n)    (((v) << n) | ((unsigned long)(v) >> (32-(n))))
#define ROR_L(v,n)    (((unsigned long)(v) >> n) | ((v) << (32-(n))))

#define VROL_L(v,n)   ((v) = ROL_L(v,n))
#define VROR_L(v,n)   ((v) = ROR_L(v,n))

#if 1
# define ASR_L(v,n)   ((v) >> (n))       /* assuming shifts are signed */
#else
# define ASR_L(v,n)   (((v) >= 0) ? ((v) >> (n)) : ~((~(v)) >> (n)))
#endif


/* Exchange two blocks of memory */
void na_exch(long *s, long *d, long size)
{
    switch (size) {

        case 0:
            return;

        case 1:
            {   register char t, *p1 = (char *)s, *p2 = (char *)d;
                t = *p1;
                *p1 = *p2;
                *p2 = t;
                return;
            }

        case 2:
            if (!(((long)s | (long)d) & 1)) {
                register short t, *p1 = (short *)s, *p2 = (short *)d;
                t = *p1;
                *p1 = *p2;
                *p2 = t;
                return;
            }
            break;

        case 4:
            if (!(((long)s | (long)d) & 3)) {
                register long *p1 = s, *p2 = d;
                size = *p1;
                *p1 = *p2;
                *p2 = size;
                return;
            }
            break;
    }

    register char t, *p1 = (char *)s, *p2 = (char *)d;
    while (--size >= 0) {
        t = *p1;
        *p1++ = *p2;
        *p2++ = t;
    }
}


long na_po2(long n)
{
    if ((unsigned long)n > 31)
        return 0;
    return 1<<n;
}


long na_log2(long n)
{
    register unsigned long un = n;
    register int v;

    for (v = -1; un != 0; v++)
		un >>= 1;
    return v;
}

/* Bit fields */

/* Bits are numbered with the MSB the longword at *v at zero. */
/* The bit field stretches from offset to (offset+size-1). */
/* This is similar to the MC68020's bit field numbering scheme. */
/* Size and offset may be any long integer values. */

#define BFCHECK(v,offset) {                 \
    if (((long)(v)) & 3) {                  \
        (offset) += 8 * (((long)(v)) & 3);  \
        (v) = (long *)(((long)(v)) & ~3);   \
    }                                       \
    v += ASR_L(offset, 5);                  \
    offset &= 31;                           \
}


/* Extract an unsigned value (up to 32 bits) from a bit-field */
long na_bfextu(long *v, long offset, long size)
{
    register long val;

    if (size <= 0) return 0;
    BFCHECK(v, offset);
    offset = 32 - offset;
    if (size <= offset)
        return ((*v) >> (offset-size)) &
               ((1 << size) - 1);
    size -= offset;
    val = ((*v++) & ((1 << offset) - 1)) << size;
    return val | ((unsigned long)(*v)) >> (32-size);
}

/* Count the number of 'one' bits in a bit-field */
long na_bfcount(long *v, long offset, long size)
{
    register long val, count = 0;

    if (size <= 0) return 0;
    while (size >= 32) {
        val = na_bfextu(v, offset, 32);
        if (~val) {
            while (val) {
                count++;
                val &= (val - 1);
            }
        } else
            count += 32;
        offset += 32;
        size -= 32;
    }
    val = na_bfextu(v, offset, size);
    while (val) {
        count++;
        val &= (val - 1);
    }
    return count;
}


/* Operations on Pascal sets */

/* Cardinality of a set */
long na_setcard(long *s)
{
    return na_bfcount(s+1, 0, *s << 5);
}

/* Miscellaneous functions */
long P_imax2(long a, long b)
{
    if (a > b)
        return a;
    else
        return b;
}

long P_imin2(long a, long b)
{
    if (a < b)
        return a;
    else
        return b;
}

void P_iswap2(long *a, long *b)
{
    long t = *a;
    *a = *b;
    *b = t;
}

long P_ibound(long min, long i, long max)
{
    if (max > min)
        return (i < min) ? min : (i > max) ? max : i;
    else
        return (i < max) ? max : (i > min) ? min : i;
}


int P_ibetween(long min, long i, long max)
{
    if (min <= i)
        return (i <= max);
    else
        return (max <= i);
}


int P_ibetweenm(long min, long i, long max, long margin)
{
    return ((min-margin <= i) && (i <= max+margin)) ||
           ((max-margin <= i) && (i <= min+margin));
}

double na_srtor(void *x)
{
  return(*((double *) &x));
}

void* na_rtosr(double x)
{
  return(*((void* *) &x));
}


